/*
 * Copyright (c) 2016. José Alberto Martín. jalbertomartinfalcon@gmail.com
 * Este proyecto ha sido realizado con la ayuda de la documentación y código de los proyectos de la
 * Dirección General de la Policía de España. Para más info: https://www.dnielectronico.es/PortalDNIe/
 * Autorizo la difusión del código fuente con fines educativos siempre que se haga referencia al
 * autor bajo los términos generales de la licencia “Academic Free License v.3.0”.
 */

package jamarfal.jalbertomartinfalcon.dnie_express;

import android.Manifest;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.design.widget.CoordinatorLayout;
import android.support.design.widget.Snackbar;
import android.support.v4.content.ContextCompat;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;


import jamarfal.jalbertomartinfalcon.dnie_express.adapter.CanAdapter;
import jamarfal.jalbertomartinfalcon.dnie_express.dialog.NewCanDialog;
import jamarfal.jalbertomartinfalcon.dnie_express.logic.Dnie;
import jamarfal.jalbertomartinfalcon.dnie_express.logic.Mrtd;
import jamarfal.jalbertomartinfalcon.dnie_express.logic.singleton.StorageSingleton;

import java.util.Vector;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;

public class CanSelectionActivity extends BaseActivity implements NewCanDialog.OnCompleteListener {


    @BindView(R.id.recyclerview_canselection_can_list)
    RecyclerView canNumberListView;
    @BindView(R.id.coordinatorlayout_canselection_wrapper)
    CoordinatorLayout rootView;


    private CanAdapter mAdapter;
    private RecyclerView.LayoutManager mLayoutManager;
    private Dnie selectedDnie;

    public static final String DNIE_EXTRA_PARCELABLE_TAG = "DNIE-PARCELABLE";
    public static final String ACTION_LABORAL_LIFE = "action_laboral_life";
    private static final String DIALOG_TAG = "new_can_dialog";

    //region Life view cycle methods
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_can_selection);

        ButterKnife.bind(this);

        initializeRecyclerView();

        Log.i("Dnie-Express", "onCreate");

    }

    @Override
    protected void onStart() {
        super.onStart();
        Log.i("Dnie-Express", "onStart");
    }

    @Override
    protected void onResume() {
        super.onResume();
        Log.i("Dnie-Express", "onResume");
    }

    @Override
    protected void onPause() {
        super.onPause();
        Log.i("Dnie-Express", "onPause");
    }

    @Override
    protected void onStop() {
        super.onStop();
        Log.i("Dnie-Express", "onStop");
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        Log.i("Dnie-Express", "onDestroy");
    }

    @Override
    protected void onNewIntent(Intent intent) {
        super.onNewIntent(intent);
        Bundle extras = intent.getExtras();
        if (extras != null) {
            boolean reloadRecyclerAdapter = extras.getBoolean(DNIeDetailActivity.RELOAD_RECYCLERVIEW_ADAPTER);
            if (reloadRecyclerAdapter) {
                mAdapter.setDnieListData(StorageSingleton.getInstance().getStorage().getAll());
            }
        }
    }

    @Override
    protected void onRestart() {
        super.onRestart();
        Log.i("Dnie-Express", "onRestart");
    }
    //endregion

    //region Menu methods
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.canselection_menu, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int id = item.getItemId();
        if (id == R.id.action_settings) {
            goToPreferencesActivity();
            return true;
        }
        return super.onOptionsItemSelected(item);
    }
    //endregion

    //region On Click methods
    @OnClick(R.id.fab)
    public void openNewCanDialog() {
        new NewCanDialog().show(getFragmentManager(), DIALOG_TAG);
    }
    //endregion

    //region OnCompleteListener Methods
    @Override
    public void onCompleteCan(String canNumber) {
        Dnie dnie = new Dnie(canNumber, null, null);
        StorageSingleton.getInstance().getStorage().add(dnie);
        mAdapter.setDnieListData(StorageSingleton.getInstance().getStorage().getAll());
        goToNfcReaderActivity(dnie, false);
    }
    //endregion

    //region Goto Methods
    private void goToNfcReaderActivity(Dnie dnie, boolean withAutentication) {
        Intent nfcReaderIntent = new Intent(this, NfcReaderActivity.class);
        nfcReaderIntent.putExtra(DNIE_EXTRA_PARCELABLE_TAG, dnie);
        nfcReaderIntent.putExtra(ACTION_LABORAL_LIFE, withAutentication);
        startActivity(nfcReaderIntent);
    }

    private void goToDnieDetailActivity(Dnie dnie) {
        Intent dnieDetailActivity = new Intent(this, DNIeDetailActivity.class);
        dnieDetailActivity.putExtra(DNIE_EXTRA_PARCELABLE_TAG, dnie);
        startActivity(dnieDetailActivity);
    }

    private void goToPreferencesActivity() {
        Intent preferencesActivity = new Intent(this, PreferencesActivity.class);
        startActivity(preferencesActivity);
    }
    //endregion

    //region Initialization Views Methods
    private void initializeRecyclerView() {
        canNumberListView.setHasFixedSize(true);

        mLayoutManager = new LinearLayoutManager(this);
        canNumberListView.setLayoutManager(mLayoutManager);

        Vector<Mrtd> dnieList = StorageSingleton.getInstance().getStorage().getAll();
        mAdapter = new CanAdapter(dnieList, new CanAdapter.OnItemClickListener() {
            @Override
            public void onItemClick(Dnie dnie) {
                selectedDnie = null;
                if (dnie.hasIncompleteData()) {
                    goToNfcReaderActivity(dnie, false);
                } else {
                    goToDnieDetailActivity(dnie);
                }
            }

            @Override
            public void onDeleteItem(Dnie dnie) {
                selectedDnie = null;
                StorageSingleton.getInstance().getStorage().remove(dnie);
                mAdapter.setDnieListData(StorageSingleton.getInstance().getStorage().getAll());
            }

            @Override
            public void onGetLaboralLife(Dnie dnie) {
                selectedDnie = dnie;
                if (ContextCompat.checkSelfPermission(CanSelectionActivity.this, Manifest.permission.WRITE_EXTERNAL_STORAGE) == PackageManager.PERMISSION_GRANTED) {
                    goToNfcReaderActivity(dnie, true);
                } else {
                    String message = getString(R.string.canselection_permission_justification_message);
                    requestPermission(Manifest.permission.WRITE_EXTERNAL_STORAGE, message, WRITE_EXTERNAL_STORAGE_PERMISSION_REQUEST, rootView);
                }
            }
        }

                , this);
        canNumberListView.setAdapter(mAdapter);
    }
    //endregion

    //region Permissions
    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        if (requestCode == WRITE_EXTERNAL_STORAGE_PERMISSION_REQUEST) {
            if (grantResults.length == 1 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                goToNfcReaderActivity(selectedDnie, true);
            } else {
                showSnackBar(getString(R.string.canselection_permission_justification_message));
            }
        }
    }
    //endregion

    @Override
    public void showSnackBar(String message) {
        Snackbar.make(rootView, message, Snackbar.LENGTH_LONG).show();
    }
}
