/*
 * Copyright (c) 2016. José Alberto Martín. jalbertomartinfalcon@gmail.com
 * Este proyecto ha sido realizado con la ayuda de la documentación y código de los proyectos de la
 * Dirección General de la Policía de España. Para más info: https://www.dnielectronico.es/PortalDNIe/
 * Autorizo la difusión del código fuente con fines educativos siempre que se haga referencia al
 * autor bajo los términos generales de la licencia “Academic Free License v.3.0”.
 */

package jamarfal.jalbertomartinfalcon.dnie_express;

import android.app.ProgressDialog;
import android.content.Intent;
import android.graphics.Bitmap;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.design.widget.Snackbar;
import android.util.Log;
import android.widget.ImageView;
import android.widget.ScrollView;
import android.widget.TextView;


import jamarfal.jalbertomartinfalcon.dnie_express.logic.Dnie;
import jamarfal.jalbertomartinfalcon.dnie_express.logic.ImageConverter;
import jamarfal.jalbertomartinfalcon.dnie_express.preferences.DnieExpressPreferences;

import butterknife.BindString;
import butterknife.BindView;
import butterknife.ButterKnife;

public class DNIeDetailActivity extends BaseActivity {

    public static final String RELOAD_RECYCLERVIEW_ADAPTER = "RELOAD_RECYCLERVIEW_ADAPTER";
    public static final String DNIEDETAIL_TAG = "DNIEDETAIL";

    @BindView(R.id.textview_dniedetail_data_name)
    TextView textViewName;
    @BindView(R.id.textview_dniedetail_data_surname)
    TextView textViewSurname;
    @BindView(R.id.textview_dniedetail_data_doc_number)
    TextView textViewDocNumber;
    @BindView(R.id.textview_dniedetail_data_doc_expiration)
    TextView textViewExpirationDate;
    @BindView(R.id.textview_dniedetail_data_birthdate)
    TextView textViewBirthDate;
    @BindView(R.id.textview_dniedetail_data_place_birth)
    TextView textViewPlaceBirth;
    @BindView(R.id.textview_dniedetail_data_country)
    TextView textViewCountry;
    @BindView(R.id.textview_dniedetail_data_address)
    TextView textViewAddress;
    @BindView(R.id.textview_dniedetail_data_city)
    TextView textViewCity;
    @BindView(R.id.imageview_dniedetail_user_image)
    ImageView userImageView;
    @BindView(R.id.imageview_dniedetail_user_sign)
    ImageView userSignImageView;
    @BindView(R.id.scrollview_dniedetail_scrollview_wrapper)
    ScrollView rootView;


    @BindString(R.string.dniedetail_converting_images)
    String convertingImages;


    private Bitmap userImage;
    private Bitmap userSignImage;
    private Dnie currentDnie;

    private DnieExpressPreferences preferences;


    //region Life Cycle
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_dnie_detail);

        currentDnie = this.getIntent().getParcelableExtra(CanSelectionActivity.DNIE_EXTRA_PARCELABLE_TAG);

        ButterKnife.bind(this);

        preferences = new DnieExpressPreferences(this);

        setDataInTextViews();


    }
    //endregion

    //region Back button
    @Override
    public void onBackPressed() {
        Intent canSelection = new Intent(this, CanSelectionActivity.class);
        canSelection.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP | Intent.FLAG_ACTIVITY_SINGLE_TOP);
        canSelection.putExtra(RELOAD_RECYCLERVIEW_ADAPTER, true);
        startActivity(canSelection);
        overridePendingTransitionExit();
    }
    //endregion

    //region Initialitation methods
    private void setDataInTextViews() {
        if (currentDnie != null) {

            if (preferences.userHasCheckedPublicData()) {
                textViewName.setText(currentDnie.getName());
                textViewSurname.setText(currentDnie.getSurname());
                textViewDocNumber.setText(currentDnie.getNif());
                textViewExpirationDate.setText(currentDnie.getDateOfExpiration());
                textViewBirthDate.setText(currentDnie.getDateOfBirth());
            }

            if (preferences.userHasCheckedPersonalData()) {
                textViewPlaceBirth.setText(currentDnie.getBirthPlace().replace("<", " (") + ")");
                textViewCountry.setText(currentDnie.getNationality());
                textViewAddress.setText(currentDnie.getAddress());
                textViewCity.setText(currentDnie.getCity());
            }


            if (preferences.userHasCheckedUserImage() || preferences.userHasCheckedSignImage()) {
                GetImagesFromBytesTask getImagesTask = new GetImagesFromBytesTask();
                getImagesTask.execute();
            }


        }
    }

    private void initializeUserImages() {
        ImageConverter imageConverter = new ImageConverter();
        try {
            userImage = imageConverter.converFromBytes(currentDnie.getPersonal_image());
            userSignImage = imageConverter.converFromBytes(currentDnie.getSign_image());
        } catch (Exception e) {
            Log.e(DNIEDETAIL_TAG, e.getMessage(), e);
            showSnackBar(getString(R.string.dniedetail_error_get_images));
        }
    }
    //endregion

    @Override
    public void showSnackBar(String message) {
        Snackbar.make(rootView, message, Snackbar.LENGTH_LONG).show();
    }


    //region Images converter task
    private class GetImagesFromBytesTask extends AsyncTask<Void, Void, Void> {

        private ProgressDialog progress;


        @Override
        protected void onPreExecute() {
            super.onPreExecute();

            progress = new ProgressDialog(DNIeDetailActivity.this);
            progress.setProgressStyle(ProgressDialog.STYLE_SPINNER);
            progress.setMessage(convertingImages);
            progress.setCancelable(false);
            progress.show();
        }

        @Override
        protected Void doInBackground(Void... voids) {
            initializeUserImages();
            return null;
        }

        @Override
        protected void onPostExecute(Void aVoid) {
            super.onPostExecute(aVoid);
            if (preferences.userHasCheckedUserImage())
                userImageView.setImageBitmap(userImage);

            if (preferences.userHasCheckedSignImage())
                userSignImageView.setImageBitmap(userSignImage);
            progress.dismiss();
        }
    }
    //endregion


}
