/*
 * Copyright (c) 2016. José Alberto Martín. jalbertomartinfalcon@gmail.com
 * Este proyecto ha sido realizado con la ayuda de la documentación y código de los proyectos de la
 * Dirección General de la Policía de España. Para más info: https://www.dnielectronico.es/PortalDNIe/
 * Autorizo la difusión del código fuente con fines educativos siempre que se haga referencia al
 * autor bajo los términos generales de la licencia “Academic Free License v.3.0”.
 */

package jamarfal.jalbertomartinfalcon.dnie_express;

import android.app.ProgressDialog;
import android.nfc.NfcAdapter;
import android.nfc.Tag;
import android.os.Bundle;
import android.support.design.widget.CoordinatorLayout;
import android.support.design.widget.Snackbar;
import android.util.Log;

import jamarfal.jalbertomartinfalcon.dnie_express.R;

import jamarfal.jalbertomartinfalcon.dnie_express.logic.Dnie;
import jamarfal.jalbertomartinfalcon.dnie_express.logic.Mrtd;
import jamarfal.jalbertomartinfalcon.dnie_express.task.AuthenticateWithDnieTask;
import jamarfal.jalbertomartinfalcon.dnie_express.task.ReadDnieDataTask;

import butterknife.BindView;
import butterknife.ButterKnife;

public class NfcReaderActivity extends BaseActivity implements NfcAdapter.ReaderCallback {

    public static final String NFC_READER_ACTIVITY_LOG = "NfcReaderActivity";
    static private NfcAdapter myNfcAdapter = null;
    ProgressDialog progressBar;
    private Mrtd currentDnie;
    private boolean readerModeON = false;
    private boolean haveToGetLaboralLife = false;
    private static final String URL_LABORAL_LIFE = "https://w2.seg-social.es/ProsaInternet/OnlineAccess?ARQ.SPM.ACTION=LOGIN&ARQ.SPM.APPTYPE=SERVICE&ARQ.IDAPP=INAF0004&INFORME=INAF0002";


    @BindView(R.id.activity_nfc_reader)
    CoordinatorLayout rootView;

    //region Life Cicle methods
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_nfc_reader);

        ButterKnife.bind(this);

        // Obtenemos el adaptador NFC
        initializeNfc();

        Bundle extras = this.getIntent().getExtras();
        if (extras != null) {
            haveToGetLaboralLife = extras.getBoolean(CanSelectionActivity.ACTION_LABORAL_LIFE);
            currentDnie = (Dnie) extras.getParcelable(CanSelectionActivity.DNIE_EXTRA_PARCELABLE_TAG);
        }

        progressBar = new ProgressDialog(NfcReaderActivity.this);

    }

    @Override
    public void onResume() {
        super.onResume();
        if (!readerModeON)
            readerModeON = EnableReaderMode(1000);
    }
    //endregion

    //region Nfc Reader Callback
    @Override
    public void onTagDiscovered(Tag tag) {
        try {
            if (haveToGetLaboralLife) {
                AuthenticateWithDnieTask authenticateWithDnieTask = new AuthenticateWithDnieTask(this, tag, progressBar, currentDnie, URL_LABORAL_LIFE);
                authenticateWithDnieTask.execute();
            } else {
                ReadDnieDataTask readDnieDataTask = new ReadDnieDataTask(this, tag, progressBar, currentDnie);
                readDnieDataTask.execute();
            }
        } catch (Exception e) {
            Log.e(NFC_READER_ACTIVITY_LOG, e.getMessage(), e);
        }
    }
    //endregion

    //region Overrided methods from parent class
    @Override
    public void showSnackBar(String message) {
        Snackbar.make(rootView, message, Snackbar.LENGTH_LONG).show();
    }
    //endregion

    private boolean EnableReaderMode(int msDelay) {
        // Ponemos en msDelay milisegundos el tiempo de espera para comprobar presencia de lectores NFC
        Bundle options = new Bundle();
        options.putInt(NfcAdapter.EXTRA_READER_PRESENCE_CHECK_DELAY, msDelay);
        myNfcAdapter.enableReaderMode(this,
                this,
                NfcAdapter.FLAG_READER_SKIP_NDEF_CHECK |
                        NfcAdapter.FLAG_READER_NFC_B,
                options);
        return true;
    }

    private boolean DisableReaderMode() {
        // Desactivamos el modo reader de NFC
        myNfcAdapter.disableReaderMode(this);
        return true;
    }

    private void initializeNfc() {
        myNfcAdapter = NfcAdapter.getDefaultAdapter(this);
        myNfcAdapter.setNdefPushMessage(null, this);
        myNfcAdapter.setNdefPushMessageCallback(null, this);
    }
}
