/*
 * Copyright (c) 2016. José Alberto Martín. jalbertomartinfalcon@gmail.com
 * Este proyecto ha sido realizado con la ayuda de la documentación y código de los proyectos de la
 * Dirección General de la Policía de España. Para más info: https://www.dnielectronico.es/PortalDNIe/
 * Autorizo la difusión del código fuente con fines educativos siempre que se haga referencia al
 * autor bajo los términos generales de la licencia “Academic Free License v.3.0”.
 */

package jamarfal.jalbertomartinfalcon.dnie_express.adapter;

import android.app.Activity;
import android.support.v7.widget.CardView;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.TextView;

import jamarfal.jalbertomartinfalcon.dnie_express.R;
import jamarfal.jalbertomartinfalcon.dnie_express.logic.Dnie;
import jamarfal.jalbertomartinfalcon.dnie_express.logic.Mrtd;

import java.util.Vector;

import butterknife.BindView;
import butterknife.ButterKnife;

/**
 * Created by jamarfal on 15/11/16.
 */

public class CanAdapter extends RecyclerView.Adapter<CanAdapter.CanHolder> {

    private Vector<Mrtd> dnieListData;
    private final OnItemClickListener listener;
    private Activity activity;

    public interface OnItemClickListener {
        void onItemClick(Dnie dnie);

        void onDeleteItem(Dnie dnie);

        void onGetLaboralLife(Dnie dnie);
    }

    public CanAdapter(Vector<Mrtd> dnieListData, OnItemClickListener listener, Activity activity) {
        this.dnieListData = dnieListData;
        this.listener = listener;
        this.activity = activity;
    }

    @Override
    public CanHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view = LayoutInflater
                .from(parent.getContext())
                .inflate(R.layout.row_canselection, parent, false);
        return new CanHolder(view);
    }

    @Override
    public void onBindViewHolder(CanHolder holder, int position) {
        Mrtd dnie = dnieListData.get(position);
        holder.bindDnie((Dnie) dnie, listener);

    }

    @Override
    public int getItemCount() {
        return dnieListData.size();
    }

    static class CanHolder extends RecyclerView.ViewHolder {
        @BindView(R.id.textview_row_canselection_cannumber)
        TextView canNumber;
        @BindView(R.id.textview_row_canselection_name)
        TextView userName;
        @BindView(R.id.textview_row_canselection_nif)
        TextView userNif;
        @BindView(R.id.cardview_row_canselection_cardview_wrapper)
        CardView cardViewWrapper;
        @BindView(R.id.button_rowcanselection_remove_can)
        Button removeCanBtn;
        @BindView(R.id.button_rowcanselection_laboral_life)
        Button laboralLifeBtn;

        CanHolder(View itemView) {
            super(itemView);
            ButterKnife.bind(this, itemView);
        }

        void bindDnie(final Dnie dnie, final OnItemClickListener listener) {
            canNumber.setText(dnie.getCan());
            userName.setText(dnie.getName());
            userNif.setText(dnie.getNif());
            cardViewWrapper.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View view) {
                    listener.onItemClick(dnie);
                }
            });

            removeCanBtn.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View view) {
                    listener.onDeleteItem(dnie);

                }
            });

            if (dnie.hasIncompleteData()) {
                laboralLifeBtn.setVisibility(View.GONE);
            } else {
                if (!laboralLifeBtn.isShown()) {
                    laboralLifeBtn.setVisibility(View.VISIBLE);
                }

                laboralLifeBtn.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View view) {
                        listener.onGetLaboralLife(dnie);
                    }
                });
            }


        }
    }

    public void setDnieListData(Vector<Mrtd> dnieListData) {
        this.dnieListData = dnieListData;
        notifyDataSetChanged();
    }
}
