/*
 * Copyright (c) 2016. José Alberto Martín. jalbertomartinfalcon@gmail.com
 * Este proyecto ha sido realizado con la ayuda de la documentación y código de los proyectos de la
 * Dirección General de la Policía de España. Para más info: https://www.dnielectronico.es/PortalDNIe/
 * Autorizo la difusión del código fuente con fines educativos siempre que se haga referencia al
 * autor bajo los términos generales de la licencia “Academic Free License v.3.0”.
 */

package jamarfal.jalbertomartinfalcon.dnie_express.logic;

import android.app.Activity;
import android.nfc.Tag;
import android.support.annotation.NonNull;
import android.util.Log;

import jamarfal.jalbertomartinfalcon.dnie_express.R;

import java.security.KeyStoreSpi;
import java.security.Security;

import de.tsenger.androsmex.mrtd.DG11;
import de.tsenger.androsmex.mrtd.DG1_Dnie;
import de.tsenger.androsmex.mrtd.DG2;
import de.tsenger.androsmex.mrtd.DG7;
import de.tsenger.androsmex.mrtd.EF_COM;
import de.tsenger.androsmex.pace.PaceException;
import es.gob.jmulticard.jse.provider.DnieKeyStore;
import es.gob.jmulticard.jse.provider.DnieProvider;
import es.gob.jmulticard.jse.provider.MrtdKeyStoreImpl;

/**
 * Created by jamarfal on 24/11/16.
 */

public class DGReader {

    private static final String ES_GOB_JMULTICARD_FASTMODE = "es.gob.jmulticard.fastmode";
    private static final String MRTD = "MRTD";
    private static final String INCORRECT_CAN = "CAN incorrecto";
    private static final String TAG_WAS_LOST = "Tag was lost";
    private Tag tagFromIntent;
    private DnieKeyStore m_ksUserMrtd;
    private boolean m_existDg1;
    private boolean m_existDg2;
    private boolean m_existDg7;
    private boolean m_existDg11;


    // Variables para leer los datos del dni. Se dividen en varios grupos.
    private DG1_Dnie m_dg1;
    private DG11 m_dg11;
    private DG2 m_dg2;
    private DG7 m_dg7;

    private Activity activity;

    private static final String DG_READER_TAG = "READ_DNIE_DGS";


    private OnUpdateProgressDialog onUpdateProgressDialog;


    public DGReader(Tag tagFromIntent, OnUpdateProgressDialog onUpdateProgressDialog, Activity activity) {
        this.tagFromIntent = tagFromIntent;
        this.onUpdateProgressDialog = onUpdateProgressDialog;
        this.activity = activity;
    }

    public Dnie loadDgs(String can) throws PaceException, Exception {

        Dnie dnie = null;

        try {
            // Leemos los datos del DG1 y DG11
            onUpdateProgressDialog.updateProgressDialog(activity.getString(R.string.dgreader_reading_data));

            // Activamos el modo rápido para agilizar la carga.
            System.setProperty(ES_GOB_JMULTICARD_FASTMODE, "true");

            // Cargamos el proveedor de servicios del DNIe
            final DnieProvider dnieProvider = loadServiceProviderForDnie(can);

            // Creamos el DnieKeyStore
            KeyStoreSpi ksSpi = new MrtdKeyStoreImpl();
            m_ksUserMrtd = new DnieKeyStore(ksSpi, dnieProvider, MRTD);
            m_ksUserMrtd.load(null, null);


            // Leemos el EF_COM para saber qué datos hay disponibles en el documento
            readEF_COM();

            // Leemos el DG1
            readDG1();


            // Leemos el DG11
            readDG11();


            // Leemos el DG2
            readDG2();

            // Leemos el DG7
            readDG7();

            // creamos el dnie
            dnie = createDnie(can);
        } catch (Exception e) {
            Log.e(DG_READER_TAG, e.getMessage(), e);
            String message = processException(e);
            throw new Exception(message);
        }

        return dnie;
    }

    @NonNull
    private String processException(Exception e) throws Exception {
        String message = activity.getString(R.string.dgreader_error_reading);
        if (e.getMessage() != null) {

            if (e.getMessage().contains(INCORRECT_CAN)) {
                message = activity.getString(R.string.dgreader_pace_error);
                throw new PaceException(message);
            }

            if (e.getMessage().contains(TAG_WAS_LOST)) {
                message += activity.getString(R.string.dgreader_connection_lost);
                throw new Exception(message);
            }
        }
        return message;
    }

    @NonNull
    private Dnie createDnie(String can) {
        Dnie dnie;
        dnie = new Dnie(can, m_dg1.getName(), m_dg1.getOptData(), m_dg1.getSurname(),
                m_dg1.getDateOfExpiry(), m_dg1.getDateOfBirth(), m_dg1.getNationality(),
                m_dg11.getBirthPlace(), m_dg11.getPersonalNumber(), m_dg11.getAddress(DG11.ADDR_DIRECCION),
                m_dg11.getAddress(DG11.ADDR_LOCALIDAD), m_dg11.getAddress(DG11.ADDR_PROVINCIA), m_dg2.getImageBytes(), m_dg7.getImageBytes());
        return dnie;
    }

    private void readDG7() {
        if (m_existDg7) {
            try {
                onUpdateProgressDialog.updateProgressDialog(activity.getString(R.string.dgreader_loading_sign));

                // Obtenemos la imagen del ciudadano
                m_dg7 = m_ksUserMrtd.getDatagroup7();
                Log.i(DG_READER_TAG, "M_DG7 cargado");
            } catch (Exception e) {
                Log.e(DG_READER_TAG, e.getMessage(), e);
                throw e;
            }
        }
    }

    private void readDG2() {
        if (m_existDg2) {
            try {
                onUpdateProgressDialog.updateProgressDialog(activity.getString(R.string.dgreader_loading_photo));

                // Obtenemos la imagen del ciudadano
                m_dg2 = m_ksUserMrtd.getDatagroup2();
                Log.i(DG_READER_TAG, "M_DG2 cargado");

            } catch (Exception e) {
                Log.e(DG_READER_TAG, e.getMessage(), e);
                throw e;
            }
        }
    }

    private void readDG11() {
        try {
            if (m_existDg11)
                m_dg11 = m_ksUserMrtd.getDatagroup11();

            Log.i(DG_READER_TAG, "M_DG11 cargado");
        } catch (Exception e) {
            Log.e(DG_READER_TAG, e.getMessage(), e);
        }
    }

    private void readDG1() {
        try {
            if (m_existDg1)
                m_dg1 = m_ksUserMrtd.getDatagroup1();

            Log.i(DG_READER_TAG, "M_DG1 cargado");
        } catch (Exception e) {
            Log.e(DG_READER_TAG, e.getMessage(), e);
        }
    }

    private void readEF_COM() {
        try {
            EF_COM m_efcom = m_ksUserMrtd.getEFCOM();
            byte[] tagList = m_efcom.getTagList();

            for (byte aTagList : tagList) {
                switch (aTagList) {
                    case 0x61:
                        // DG_1. Lo leemos siempre que esté disponible
                        m_existDg1 = true;
                        break;
                    case 0x75:
                        // DG_2. Lo leemos si el usuario lo especificó
                        m_existDg2 = true;
                        break;
                    case 0x67:
                        // DG_7. Lo leemos si el usuario lo especificó
                        m_existDg7 = true;
                        break;
                    case 0x6B:
                        // DG_11. Lo leemos siempre que esté disponible
                        m_existDg11 = true;
                        break;
                }
            }
        } catch (Exception e) {
            Log.e(DG_READER_TAG, e.getMessage(), e);
            throw e;
        }

        Log.i(DG_READER_TAG, "LEIDO EL EF_COM");
    }

    @NonNull
    private DnieProvider loadServiceProviderForDnie(String can) {
        final DnieProvider p = new DnieProvider();
        p.setProviderTag(tagFromIntent);
        String can6digitos = can;
        while (can6digitos.length() < 6)
            can6digitos = "0" + can6digitos;
        p.setProviderCan(can6digitos);
        Security.insertProviderAt(p, 1);
        return p;
    }
}
