/*
 * Copyright (c) 2016. José Alberto Martín. jalbertomartinfalcon@gmail.com
 * Este proyecto ha sido realizado con la ayuda de la documentación y código de los proyectos de la
 * Dirección General de la Policía de España. Para más info: https://www.dnielectronico.es/PortalDNIe/
 * Autorizo la difusión del código fuente con fines educativos siempre que se haga referencia al
 * autor bajo los términos generales de la licencia “Academic Free License v.3.0”.
 */

package jamarfal.jalbertomartinfalcon.dnie_express.logic;

import android.app.Activity;
import android.nfc.Tag;
import android.util.Log;

import jamarfal.jalbertomartinfalcon.dnie_express.R;
import jamarfal.jalbertomartinfalcon.dnie_express.dialog.RequestPinDialog;

import java.security.KeyStoreSpi;
import java.security.Security;

import de.tsenger.androsmex.pace.PaceException;
import es.gob.jmulticard.jse.provider.DnieKeyStore;
import es.gob.jmulticard.jse.provider.DnieProvider;
import es.gob.jmulticard.jse.provider.MrtdKeyStoreImpl;
import es.gob.jmulticard.ui.passwordcallback.DNIeDialogManager;

/**
 * Created by jamarfal on 24/11/16.
 */

public class DGReaderWithPin {

    private static final String ES_GOB_JMULTICARD_FASTMODE = "es.gob.jmulticard.fastmode";
    private static final String MRTD = "MRTD";
    private static final String INCORRECT_CAN = "CAN incorrecto";
    private static final String TAG_WAS_LOST = "Tag was lost";
    private static final String DG_READER_TAG = "READ_DNIE_DGS";
    private static final String AUTH_CERT_ALIAS = "CertAutenticacion"; //$NON-NLS-1$
    private Tag tagFromIntent;
    private Activity activity;
    private OnUpdateProgressDialog onUpdateProgressDialog;


    public DGReaderWithPin(Tag tagFromIntent, OnUpdateProgressDialog onUpdateProgressDialog, Activity activity) {
        this.tagFromIntent = tagFromIntent;
        this.onUpdateProgressDialog = onUpdateProgressDialog;
        this.activity = activity;
    }

    public DnieKeyStore loadDgs(String can) throws PaceException, Exception {
        DnieKeyStore dnieKeyStore = null;

        try {
            // Leemos los datos del DG1 y DG11
            onUpdateProgressDialog.updateProgressDialog(activity.getString(R.string.dgreader_reading_data));

            // Activamos el modo rápido para agilizar la carga.
            System.setProperty(ES_GOB_JMULTICARD_FASTMODE, "true");

            // Cargamos el proveedor de servicios del DNIe
            final DnieProvider dnieProvider = loadServiceProviderForDnie(can);

            // Creamos el DnieKeyStore
            KeyStoreSpi ksSpi = new MrtdKeyStoreImpl();
            dnieKeyStore = new DnieKeyStore(ksSpi, dnieProvider, MRTD);
            dnieKeyStore.load(null, null);

            // Construímos el diálogo que solicitará el PIN
            RequestPinDialog requestPinDialog = new RequestPinDialog(activity, true);
            DNIeDialogManager.setDialogUIHandler(requestPinDialog);

            onUpdateProgressDialog.updateProgressDialog(activity.getString(R.string.dgreaderpin_loading_certificates));

            // Forzamos a cargar los certificados reales pidiendo la clave
            dnieKeyStore.getKey(AUTH_CERT_ALIAS, null);
            dnieKeyStore.getCertificate(AUTH_CERT_ALIAS).getEncoded();

        } catch (Exception e) {
            Log.e(DG_READER_TAG, e.getMessage(), e);
            String message = processException(e);
            throw new Exception(message);
        }
        return dnieKeyStore;
    }

    private DnieProvider loadServiceProviderForDnie(String can) {
        final DnieProvider dnieProvider = new DnieProvider();
        dnieProvider.setProviderTag(tagFromIntent);
        String can6digitos = can;
        while (can6digitos.length() < 6)
            can6digitos = "0" + can6digitos;
        dnieProvider.setProviderCan(can6digitos);
        Security.insertProviderAt(dnieProvider, 1);
        return dnieProvider;
    }


    private String processException(Exception e) throws Exception {
        String message = activity.getString(R.string.dgreader_error_reading);
        if (e.getMessage() != null) {

            if (e.getMessage().contains(INCORRECT_CAN)) {
                message = activity.getString(R.string.dgreader_pace_error);
                throw new PaceException(message);
            }

            if (e.getMessage().contains(TAG_WAS_LOST)) {
                message += activity.getString(R.string.dgreader_connection_lost);
                throw new Exception(message);
            }

        }
        return message;
    }
}
