/*
 * Copyright (c) 2016. José Alberto Martín. jalbertomartinfalcon@gmail.com
 * Este proyecto ha sido realizado con la ayuda de la documentación y código de los proyectos de la
 * Dirección General de la Policía de España. Para más info: https://www.dnielectronico.es/PortalDNIe/
 * Autorizo la difusión del código fuente con fines educativos siempre que se haga referencia al
 * autor bajo los términos generales de la licencia “Academic Free License v.3.0”.
 */

package jamarfal.jalbertomartinfalcon.dnie_express.logic;

/**
 * Created by jamarfal on 16/11/16.
 */

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;

import java.util.Vector;


/**
 * Created by jamarfal on 14/11/16.
 */

public class DatabaseHelperStorage extends SQLiteOpenHelper implements Storage {

    protected Context mContext;
    private static final String LOG = "DatabaseHelperStorage";
    private static final int DATABASE_VERSION = 1;
    private static final String DATABASE_NAME = "dnieexpress";
    private static final String TABLE_CAN = "can";
    private static final String KEY_CAN = "can_number";
    private static final String KEY_NAME = "user_name";
    private static final String KEY_SURNAME = "user_surname";
    private static final String KEY_NIF = "user_nif";
    private static final String KEY_EXPIRATION = "expiration_date";
    private static final String KEY_BIRTHDATE = "birth_date";
    private static final String KEY_NATIONALITY = "nationality";
    private static final String KEY_BIRTH_PLACE = "birth_place";
    private static final String KEY_PERSONAL_NUMBER = "personal_number";
    private static final String KEY_ADDRESS = "address";
    private static final String KEY_CITY = "city";
    private static final String KEY_PROVINCE = "province";
    private static final String KEY_USER_IMAGE = "user_image";
    private static final String KEY_USER_SIGN_IMAGE = "user_sign_image";
    private static final String CREATE_TABLE_CAN =
            "CREATE TABLE " + TABLE_CAN + " (" +
                    KEY_CAN + " CHARACTER(6) NOT NULL PRIMARY KEY, " +
                    KEY_NAME + " CHARACTER(255) NULL, " +
                    KEY_SURNAME + " CHARACTER(255) NULL, " +
                    KEY_NIF + " CHARACTER(120) NULL, " +
                    KEY_EXPIRATION + " TIMESTAMP NULL, " +
                    KEY_BIRTHDATE + " TIMESTAMP NULL, " +
                    KEY_NATIONALITY + " CHARACTER(120) NULL, " +
                    KEY_BIRTH_PLACE + " CHARACTER(120) NULL, " +
                    KEY_PERSONAL_NUMBER + " CHARACTER(120) NULL, " +
                    KEY_ADDRESS + " CHARACTER(120) NULL, " +
                    KEY_CITY + " CHARACTER(120) NULL, " +
                    KEY_PROVINCE + " CHARACTER(120) NULL," +
                    KEY_USER_IMAGE + " BLOB NULL," +
                    KEY_USER_SIGN_IMAGE + " BLOB NULL" +
                    ");";

    public DatabaseHelperStorage(Context context) {
        super(context, DATABASE_NAME, null, DATABASE_VERSION);
        mContext = context;
    }


    @Override
    public void onCreate(SQLiteDatabase db) {
        db.execSQL(CREATE_TABLE_CAN);
    }

    @Override
    public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
        db.execSQL("DROP TABLE IF EXISTS " + TABLE_CAN);
        onCreate(db);
    }

    @Override
    public void add(Mrtd mrtd) {
        Dnie dnie = (Dnie) mrtd;
        SQLiteDatabase db = getWritableDatabase();
        ContentValues values = new ContentValues();
        values.put(KEY_CAN, dnie.getCan());
        values.put(KEY_NAME, dnie.getName());
        values.put(KEY_SURNAME, dnie.getSurname());
        values.put(KEY_NIF, dnie.getNif());
        values.put(KEY_EXPIRATION, dnie.getDateOfExpiration());
        values.put(KEY_BIRTHDATE, dnie.getDateOfBirth());
        values.put(KEY_NATIONALITY, dnie.getNationality());
        values.put(KEY_BIRTH_PLACE, dnie.getBirthPlace());
        values.put(KEY_PERSONAL_NUMBER, dnie.getPersonalNumber());
        values.put(KEY_ADDRESS, dnie.getAddress());
        values.put(KEY_CITY, dnie.getCity());
        values.put(KEY_PROVINCE, dnie.getProvince());
        values.put(KEY_USER_IMAGE, dnie.getPersonal_image());
        values.put(KEY_USER_SIGN_IMAGE, dnie.getSign_image());

        db.insert(TABLE_CAN, null, values);
        db.close();
    }

    @Override
    public void remove(String can) {
        SQLiteDatabase db = getWritableDatabase();
        String where = KEY_CAN + "='?'";
        String[] parameters = new String[]{can};
        db.delete(TABLE_CAN, where, parameters);
        db.close();
    }

    @Override
    public void remove(Mrtd mrtd) {
        SQLiteDatabase db = getWritableDatabase();
        String where = KEY_CAN + "=?";
        String[] parameters = new String[]{mrtd.getCan()};
        db.delete(TABLE_CAN, where, parameters);
        db.close();
    }

    @Override
    public void update(String can) {

    }

    @Override
    public void update(Mrtd mrtd) {
        Dnie dnie = (Dnie) mrtd;
        SQLiteDatabase db = getWritableDatabase();
        ContentValues values = new ContentValues();
        values.put(KEY_CAN, dnie.getCan());
        values.put(KEY_NAME, dnie.getName());
        values.put(KEY_SURNAME, dnie.getSurname());
        values.put(KEY_NIF, dnie.getNif());
        values.put(KEY_EXPIRATION, dnie.getDateOfExpiration());
        values.put(KEY_BIRTHDATE, dnie.getDateOfBirth());
        values.put(KEY_NATIONALITY, dnie.getNationality());
        values.put(KEY_BIRTH_PLACE, dnie.getBirthPlace());
        values.put(KEY_PERSONAL_NUMBER, dnie.getPersonalNumber());
        values.put(KEY_ADDRESS, dnie.getAddress());
        values.put(KEY_CITY, dnie.getCity());
        values.put(KEY_PROVINCE, dnie.getProvince());
        values.put(KEY_USER_IMAGE, dnie.getPersonal_image());
        values.put(KEY_USER_SIGN_IMAGE, dnie.getSign_image());

        db.update(TABLE_CAN, values, KEY_CAN + " = ?", new String[]{dnie.getCan()});
        db.close();
    }

    @Override
    public Mrtd search(String can) {
        Dnie dnie = null;
        SQLiteDatabase db = getReadableDatabase();
        String query = "SELECT * FROM " + TABLE_CAN + "WHERE " + KEY_CAN + "=" + can;
        Cursor cursor = db.rawQuery(query, null);
        if (cursor.moveToNext()) {
            dnie = getDnieObjectFromCursor(cursor);
        }
        cursor.close();
        db.close();
        return dnie;
    }

    @Override
    public Vector<Mrtd> getAll() {
        Vector<Mrtd> mrtdList = new Vector<>();
        SQLiteDatabase db = getReadableDatabase();
        String query = "SELECT * FROM " + TABLE_CAN;
        Cursor cursor = db.rawQuery(query, null);
        if (cursor.moveToFirst()) {
            do {
                Dnie dnie = getDnieObjectFromCursor(cursor);
                mrtdList.add(dnie);
            } while (cursor.moveToNext());
        }
        cursor.close();
        db.close();
        return mrtdList;
    }

    @Override
    public int size() {
        return 0;
    }

    private Dnie getDnieObjectFromCursor(Cursor cursor) {
        Dnie dnie = new Dnie();
        dnie.setCan(cursor.getString(cursor.getColumnIndex(KEY_CAN)));
        dnie.setName(cursor.getString(cursor.getColumnIndex(KEY_NAME)));
        dnie.setSurname(cursor.getString(cursor.getColumnIndex(KEY_SURNAME)));
        dnie.setNif(cursor.getString(cursor.getColumnIndex(KEY_NIF)));
        dnie.setDateOfExpiration(cursor.getString(cursor.getColumnIndex(KEY_EXPIRATION)));
        dnie.setDateOfBirth(cursor.getString(cursor.getColumnIndex(KEY_BIRTHDATE)));
        dnie.setNationality(cursor.getString(cursor.getColumnIndex(KEY_NATIONALITY)));
        dnie.setBirthPlace(cursor.getString(cursor.getColumnIndex(KEY_BIRTH_PLACE)));
        dnie.setPersonalNumber(cursor.getString(cursor.getColumnIndex(KEY_PERSONAL_NUMBER)));
        dnie.setAddress(cursor.getString(cursor.getColumnIndex(KEY_ADDRESS)));
        dnie.setCity(cursor.getString(cursor.getColumnIndex(KEY_CITY)));
        dnie.setProvince(cursor.getString(cursor.getColumnIndex(KEY_PROVINCE)));
        dnie.setPersonal_image(cursor.getBlob(cursor.getColumnIndex(KEY_USER_IMAGE)));
        dnie.setSign_image(cursor.getBlob(cursor.getColumnIndex(KEY_USER_SIGN_IMAGE)));
        return dnie;
    }
}


