/*
 * Copyright (c) 2016. José Alberto Martín. jalbertomartinfalcon@gmail.com
 * Este proyecto ha sido realizado con la ayuda de la documentación y código de los proyectos de la
 * Dirección General de la Policía de España. Para más info: https://www.dnielectronico.es/PortalDNIe/
 * Autorizo la difusión del código fuente con fines educativos siempre que se haga referencia al
 * autor bajo los términos generales de la licencia “Academic Free License v.3.0”.
 */

package jamarfal.jalbertomartinfalcon.dnie_express.logic;

import android.app.DownloadManager;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.support.annotation.NonNull;
import android.util.Log;

import org.apache.http.HttpEntity;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.util.EntityUtils;
import jamarfal.jalbertomartinfalcon.dnie_express.R;

import java.io.File;
import java.io.IOException;
import java.net.SocketTimeoutException;
import java.nio.charset.Charset;

import es.gob.jmulticard.jse.provider.DnieKeyStore;
import es.inteco.labs.android.utils.FileUtils;
import es.inteco.labs.net.DNIeCaCertsManager;
import es.inteco.labs.net.DroidHttpClient;

/**
 * Created by jamarfal on 7/12/16.
 */

public class SecureConnectionWithDnie {

    private static final String SEG_SOCIAL_URL_BASE = "https://w2.seg-social.es";
    private static final String LABORAL_LIFE_DOC_NAME = "Vida_Laboral";
    private static final String TYPE = "pdf";
    private static final String SECURECONNECTION_TAG = "SECURECONNECTION";
    public static final String APPLICATION_PDF = "application/pdf";
    public static final String TITLE = "Informe de Vida Laboral";
    private OnUpdateProgressDialog updateProgressDialog;
    private Context activity;
    private String sslUrl;
    private DnieKeyStore dnieKeyStore;

    public SecureConnectionWithDnie(OnUpdateProgressDialog updateProgressDialog, Context activity, DnieKeyStore dnieKeyStore, String sslUrl) {
        this.updateProgressDialog = updateProgressDialog;
        this.activity = activity;
        this.dnieKeyStore = dnieKeyStore;
        this.sslUrl = sslUrl;

    }

    public boolean ConectarSSL() throws Exception {
        String resultText;
        File sharedFile;

        try {
            // Procedemos a la conexión con el servidor
            updateProgressDialog.updateProgressDialog(activity.getString(R.string.secureconnection_throwing_connection));

            // Creamos el objeto que gestionará los certificados adicionales
            MyCaCertificate myCaCertificate = new MyCaCertificate(activity);
            DNIeCaCertsManager.setCaCertHandler(myCaCertificate);


            // Ejecuta la primera parte de la petición para conectarse de forma segura
            HttpEntity webContent = DroidHttpClient.executeRequest(sslUrl, activity, dnieKeyStore);

            // Se obtiene la segunda URL para obtener el documento
            String encoding = getEncoding(webContent);
            String finalUrl = getCompleteLaboralLifeURL(webContent, encoding);

            updateProgressDialog.updateProgressDialog(activity.getString(R.string.secureconnection_creating_document));

            // Ejecuta la segunda petición para obtener el documento
            webContent = DroidHttpClient.executeRequest(finalUrl, activity, this.dnieKeyStore);

            String mimeType = DroidHttpClient.getContentType(webContent);
            if (mimeType.toLowerCase().startsWith(APPLICATION_PDF)) {
                updateProgressDialog.updateProgressDialog(activity.getString(R.string.secureconnection_dowloading_document));

                sharedFile = FileUtils.writeDownloadFile(LABORAL_LIFE_DOC_NAME, TYPE, webContent.getContent());
                ((DownloadManager) activity.getApplicationContext().getSystemService(Context.DOWNLOAD_SERVICE)).addCompletedDownload(sharedFile.getName(), TITLE, true, mimeType, sharedFile.getAbsolutePath(), sharedFile.length(), true);

                // Selección del programa para abrir el pdf
                createPdfChooser(sharedFile, mimeType);

                DroidHttpClient.cleanCookies();
                return true;
            } else {
                resultText = activity.getString(R.string.secureconnection_cannot_dowloading);
                throw new Exception(resultText);
            }


        } catch (ClientProtocolException e) {
            resultText = activity.getString(R.string.secureconnection_cannot_set_connection);
            throw new ClientProtocolException(resultText);
        } catch (SocketTimeoutException e) {
            resultText = activity.getString(R.string.secureconnection_timeout);
            throw new SocketTimeoutException(resultText);
        } catch (IOException e) {
            resultText = activity.getString(R.string.securecconnection_internet_problem)
                    + ((e.getMessage() != null) ? ("\n" + e.getMessage()) : "");
            throw new IOException(resultText);
        } catch (Exception e) {
            Log.e(SECURECONNECTION_TAG, e.getMessage(), e);
            resultText = activity.getString(R.string.secureconnection_ssl_problem);
            throw new Exception(resultText);
        }

    }

    private void createPdfChooser(File sharedFile, String mimeType) {
        Intent target = new Intent(Intent.ACTION_VIEW);
        target.setDataAndType(Uri.fromFile(sharedFile), mimeType.toLowerCase());
        target.setFlags(Intent.FLAG_ACTIVITY_NO_HISTORY);
        Intent intent = Intent.createChooser(target, "Open File");
        activity.startActivity(intent);
    }

    private String getCompleteLaboralLifeURL(HttpEntity webContent, String encoding) throws IOException {
        String sslResult = new String(EntityUtils.toByteArray(webContent), Charset.forName(encoding));

        int iStart = sslResult.indexOf("href=\"/ProsaInternet") + 6;

        return (SEG_SOCIAL_URL_BASE + sslResult.substring(iStart, sslResult.indexOf("=IMPRIMIBLE\"", iStart) + 11)).replace(" ", "").replace("&amp;", "&");
    }

    @NonNull
    private String getEncoding(HttpEntity webContent) {
        String codificacion = EntityUtils.getContentCharSet(webContent);
        codificacion = (codificacion == null) ? "utf-8" : codificacion;
        return codificacion;
    }
}
