/*
 * Copyright (c) 2016. José Alberto Martín. jalbertomartinfalcon@gmail.com
 * Este proyecto ha sido realizado con la ayuda de la documentación y código de los proyectos de la
 * Dirección General de la Policía de España. Para más info: https://www.dnielectronico.es/PortalDNIe/
 * Autorizo la difusión del código fuente con fines educativos siempre que se haga referencia al
 * autor bajo los términos generales de la licencia “Academic Free License v.3.0”.
 */

package jamarfal.jalbertomartinfalcon.dnie_express.task;

import android.app.Activity;
import android.app.ProgressDialog;
import android.nfc.Tag;
import android.os.AsyncTask;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;

import jamarfal.jalbertomartinfalcon.dnie_express.BaseActivity;
import jamarfal.jalbertomartinfalcon.dnie_express.R;
import jamarfal.jalbertomartinfalcon.dnie_express.logic.DGReaderWithPin;
import jamarfal.jalbertomartinfalcon.dnie_express.logic.Dnie;
import jamarfal.jalbertomartinfalcon.dnie_express.logic.Mrtd;
import jamarfal.jalbertomartinfalcon.dnie_express.logic.OnUpdateProgressDialog;
import jamarfal.jalbertomartinfalcon.dnie_express.logic.SecureConnectionWithDnie;

import es.gob.jmulticard.jse.provider.DnieKeyStore;

/**
 * Created by jamarfal on 24/11/16.
 */

public class AuthenticateWithDnieTask extends AsyncTask<Void, Integer, Void> implements OnUpdateProgressDialog {

    private static final String AUTHTENTICATION_WITH_DNIE_TASK_LOG = "AUTH_DNIE_TASK_LOG";

    private ProgressDialog progressDialog;
    private Activity activity;
    private Tag tagFromIntent;
    private Dnie currentDnie;
    private Exception exceptionToBeThrown;
    final Handler myHandler = new Handler(Looper.getMainLooper());
    final Runnable updateStatus = new Runnable() {
        public void run() {
            progressDialog.setMessage(progressDialogMessage);
            if (!progressDialog.isShowing())
                progressDialog.show();
        }
    };

    private String sslUrl;

    private String progressDialogMessage;

    public AuthenticateWithDnieTask(Activity activity, Tag tagFromIntent, ProgressDialog progressDialog, Mrtd currentDnie, String sslUrl) {
        this.activity = activity;
        this.tagFromIntent = tagFromIntent;
        this.progressDialog = progressDialog;
        this.currentDnie = (Dnie) currentDnie;
        this.sslUrl = sslUrl;
    }

    //region Asynctask methods
    @Override
    protected void onPreExecute() {
        super.onPreExecute();
        progressDialog.setIndeterminate(true);
        progressDialog.setCancelable(false);
        progressDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        progressDialog.setTitle(R.string.app_name);
    }

    @Override
    protected Void doInBackground(Void... voids) {
        DnieKeyStore dnieKeyStore;
        try {
            // PASO 1: Leemos el CDF para obtener los certificados almacenados en la tarjeta
            DGReaderWithPin dgReaderWithPin = new DGReaderWithPin(tagFromIntent, this, activity);
            dnieKeyStore = dgReaderWithPin.loadDgs(currentDnie.getCan());

            // PASO 2: Lanzamos la conexión SSL
            SecureConnectionWithDnie secureConnectionWithDnie = new SecureConnectionWithDnie(this, activity, dnieKeyStore, sslUrl);
            secureConnectionWithDnie.ConectarSSL();

        } catch (Exception e) {
            Log.e(AUTHTENTICATION_WITH_DNIE_TASK_LOG, e.getMessage(), e);
            exceptionToBeThrown = e;
        }
        return null;
    }

    @Override
    protected void onPostExecute(Void aVoid) {
        // Destruimos el cuadro de diálogo
        progressDialog.dismiss();
        if (exceptionToBeThrown != null) {
            ((BaseActivity) activity).showSnackBar(exceptionToBeThrown.getMessage());
        }
    }
    //endregion

    @Override
    public void updateProgressDialog(String text) {
        progressDialogMessage = text;
        myHandler.post(updateStatus);
    }
}
