/*
 * Copyright (c) 2016. José Alberto Martín. jalbertomartinfalcon@gmail.com
 * Este proyecto ha sido realizado con la ayuda de la documentación y código de los proyectos de la
 * Dirección General de la Policía de España. Para más info: https://www.dnielectronico.es/PortalDNIe/
 * Autorizo la difusión del código fuente con fines educativos siempre que se haga referencia al
 * autor bajo los términos generales de la licencia “Academic Free License v.3.0”.
 */

package jamarfal.jalbertomartinfalcon.dnie_express.task;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Intent;
import android.nfc.Tag;
import android.os.AsyncTask;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;

import jamarfal.jalbertomartinfalcon.dnie_express.BaseActivity;
import jamarfal.jalbertomartinfalcon.dnie_express.CanSelectionActivity;
import jamarfal.jalbertomartinfalcon.dnie_express.DNIeDetailActivity;
import jamarfal.jalbertomartinfalcon.dnie_express.R;
import jamarfal.jalbertomartinfalcon.dnie_express.logic.DGReader;
import jamarfal.jalbertomartinfalcon.dnie_express.logic.DatabaseHelperStorage;
import jamarfal.jalbertomartinfalcon.dnie_express.logic.Dnie;
import jamarfal.jalbertomartinfalcon.dnie_express.logic.Mrtd;
import jamarfal.jalbertomartinfalcon.dnie_express.logic.OnUpdateProgressDialog;

import de.tsenger.androsmex.pace.PaceException;

/**
 * Created by jamarfal on 24/11/16.
 */

public class ReadDnieDataTask extends AsyncTask<Void, Integer, Dnie> implements OnUpdateProgressDialog {

    private static final String READ_DNIE_DATA_TASK_LOG = "READ_DNIE_DATA_TASK_LOG";
    private static final String LOST = "lost";

    private ProgressDialog progressDialog;
    private Activity activity;
    private Tag tagFromIntent;
    private Dnie currentDnie;
    final Handler myHandler = new Handler(Looper.getMainLooper());
    final Runnable updateStatus = new Runnable() {
        public void run() {
            progressDialog.setMessage(progressDialogMessage);
            if (!progressDialog.isShowing())
                progressDialog.show();
        }
    };

    private Exception exceptionToBeThrown;

    private String progressDialogMessage;


    public ReadDnieDataTask(Activity activity, Tag tagFromIntent, ProgressDialog progressDialog, Mrtd currentDnie) {
        this.activity = activity;
        this.tagFromIntent = tagFromIntent;
        this.progressDialog = progressDialog;
        this.currentDnie = (Dnie) currentDnie;
    }

    @Override
    protected void onPreExecute() {
        super.onPreExecute();
        progressDialog.setIndeterminate(true);
        progressDialog.setCancelable(false);
        progressDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
        progressDialog.setTitle(R.string.app_name);
    }

    @Override
    protected Dnie doInBackground(Void... voids) {
        // Se leen los datos públicos del documento
        Dnie dnie;
        try {
            // Lanzamos la operación de lectura del DNIe
            DGReader dgReader = new DGReader(tagFromIntent, this, activity);
            // carga los DGs del dnie para leer los datos públicos
            dnie = dgReader.loadDgs(currentDnie.getCan());
        } catch (PaceException e) {
            Log.e(READ_DNIE_DATA_TASK_LOG, e.getMessage(), e);
            exceptionToBeThrown = e;
            return null;
        } catch (Exception e) {
            Log.e(READ_DNIE_DATA_TASK_LOG, e.getMessage(), e);
            exceptionToBeThrown = e;
            return null;
        }

        return dnie;
    }

    @Override
    protected void onPostExecute(Dnie dnie) {
        // Destruimos el cuadro de diálogo
        progressDialog.dismiss();

        if (dnie != null) {
            DatabaseHelperStorage databaseHelperStorage = new DatabaseHelperStorage(activity);
            databaseHelperStorage.update(dnie);
            Intent dnieDetailIntent = new Intent(activity.getApplicationContext(), DNIeDetailActivity.class);
            dnieDetailIntent.putExtra(CanSelectionActivity.DNIE_EXTRA_PARCELABLE_TAG, dnie);
            activity.startActivity(dnieDetailIntent);
        } else {
            if (exceptionToBeThrown != null) {
                ((BaseActivity) activity).showSnackBar(exceptionToBeThrown.getMessage());
            } else {
                ((BaseActivity) activity).showSnackBar(activity.getString(R.string.readdnietask_error_conversion_data));
            }
        }

        Log.i(READ_DNIE_DATA_TASK_LOG, "Termina la operación de lectura");
    }


    @Override
    public void updateProgressDialog(String text) {
        progressDialogMessage = text;
        myHandler.post(updateStatus);
    }
}
